from setuptools import setup

setup(name='streampy',
      version='1.0',
      description='Java like stream pipelines, supports parallel operations',
      url='https://github.com/tolsac/streampy.git',
      author='Camille Tolsa',
      author_email='camille.tolsa@gmail.com',
      license='MIT',
      packages=['.'],
      zip_safe=False)
